'use strict';

exports.__esModule = true;
exports.FileOutput = undefined;
exports.default = createFileOutput;

var _stream = require('stream');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _dateformat = require('./dateformat');

var _dateformat2 = _interopRequireDefault(_dateformat);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FileOutput = exports.FileOutput = function (_Writable) {
  _inherits(FileOutput, _Writable);

  function FileOutput() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
      interval: 600,
      maxSize: 1024 * 1024,
      name: '%Y-%m-%d-%H-%M-%r-log.log',
      codec: null,
      dir: './logs/'
    };

    _classCallCheck(this, FileOutput);

    var _this = _possibleConstructorReturn(this, _Writable.call(this, {
      objectMode: config.codec != null
    }));
    // If codec is specified, we'd be using object mode.


    _this.name = config.name || '%Y-%m-%d-%H-%M-%R-log.log';
    _this.interval = config.interval || 600;
    _this.maxSize = config.maxSize || 1024 * 1024;
    _this.codec = config.codec;
    _this.dir = config.dir || './logs/';

    // If the name is not fixed, we have to carry multiple file stream
    // at the single time. However, since the interval is fixed, we don't need
    // to care about the data per each file.
    // Name must be fixed if codec is not provided, though.
    _this.isDynamic = typeof _this.name === 'function';
    // Record currently ingested file size.
    _this.currentSize = 0;
    // Create file swapper worker.
    _mkdirp2.default.sync(_this.dir);
    _this.refresher = setInterval(_this.refresh.bind(_this), 1000);
    _this.refresh(true);
    return _this;
  }

  FileOutput.prototype.refresh = function refresh(force, callback) {
    if (!(this.currentSize > this.maxSize || Date.now() - this.refreshedAt > this.interval * 1000 || force)) return;
    this.refreshedAt = Date.now();
    this.currentSize = 0;
    // Finish the ingestion, if any.
    if (this.isDynamic) {
      // TODO Add async.all?
      var remainingCount = 0;
      var innerCallback = function innerCallback() {
        remainingCount--;
        if (remainingCount === 0 && callback) callback();
      };
      for (var key in this.files) {
        remainingCount++;
        this.finalizeFile(this.files[key], innerCallback);
      }
      this.files = {};
      if (remainingCount === 0 && callback) callback();
    } else if (this.file != null) {
      this.finalizeFile(this.file, callback);
      // Create new file
      this.file = null;
    } else {
      if (callback) callback();
    }
  };

  FileOutput.prototype.getFileName = function getFileName(nameTemplate) {
    var date = new Date(this.refreshedAt);
    // Format the name to fill in the details, such as date and random id.
    var name = (0, _dateformat2.default)(date, nameTemplate);
    name = name.replace(/%r/g, function () {
      return Math.random() * 1e8 | 0;
    });
    return _path2.default.resolve(this.dir, name);
  };

  FileOutput.prototype.createFile = function createFile(nameTemplate) {
    var _this2 = this;

    // TODO Add mkdirp. This may not be possible since fs stream must be
    // initialized synchronously.
    var fileStream = _fs2.default.createWriteStream(this.getFileName(nameTemplate), { flags: 'a' });
    var stream = fileStream;
    // Before putting the data to file stream, we need to calculate how much
    // data we've used. We'll wrap a transform stream for that.
    var counterStream = new _stream.Transform({
      transform: function transform(chunk, encoding, callback) {
        if (chunk instanceof Buffer) {
          _this2.currentSize += chunk.length;
          callback(null, chunk);
        } else {
          var buffer = Buffer.from(chunk, encoding);
          _this2.currentSize += buffer.length;
          callback(null, buffer);
        }
      }
    });
    // If codec is specified, create a passthrough stream and call codec with
    // it, then pipe the returned value of codec to file stream.
    if (this.codec) {
      stream = new _stream.PassThrough({ objectMode: true });
      this.codec(stream).pipe(counterStream).pipe(fileStream);
    } else {
      stream = counterStream;
      counterStream.pipe(fileStream);
    }
    return {
      stream: stream,
      fileStream: fileStream
    };
  };

  FileOutput.prototype.finalizeFile = function finalizeFile(file, callback) {
    // Finalize the stream and end the data ingestion, then send to AWS,
    // then delete the file.
    file.stream.end();
    file.fileStream.once('close', callback);
  };

  FileOutput.prototype._write = function _write(chunk, encoding, callback) {
    // We need to honor the backpressure, thus we're forwarding the callback.
    this.refresh();
    if (this.isDynamic) {
      var name = this.name(chunk);
      var file = this.files[name];
      if (file == null) {
        file = this.files[name] = this.createFile(name);
      }
      file.stream.write(chunk, encoding, callback);
    } else {
      if (this.file == null) {
        this.file = this.createFile(this.name);
      }
      this.file.stream.write(chunk, encoding, callback);
    }
  };

  FileOutput.prototype._final = function _final(callback) {
    clearInterval(this.refresher);
    this.refresh(true, callback);
  };

  return FileOutput;
}(_stream.Writable);

function createFileOutput(config) {
  return new FileOutput(config);
}

module.exports = createFileOutput;
createFileOutput.FileOutput = FileOutput;